@@ -1,29 +1,7 @@
-/* vim: set sw=4 ts=4 sts=4 et: */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <sys/mman.h>
-#include <unistd.h>
-#include <time.h>
-#include <fcntl.h>
-#include <fnmatch.h>
-
-#ifdef _WIN32
-# include <winsock2.h>
-#endif
-
-#ifdef HAVE_ARPA_INET_H
-# include <arpa/inet.h>
-#endif
-
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
 #elif defined __GNUC__
@@ -41,11 +19,32 @@ extern "C"
 void *alloca (size_t);
 #endif
 
+#include <stdio.h>
+#include <string.h>
+#include <ctype.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/mman.h>
+#include <unistd.h>
+#include <time.h>
+#include <fcntl.h>
+#include <fnmatch.h>
+#include <limits.h>
+
+#ifdef _WIN32
+# include <winsock2.h>
+#endif
+
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
+
 #include <Ecore.h>
 #include <Ecore_File.h>
 
-#include <Efreet.h>
-#include <Efreet_Mime.h>
+#include "Efreet.h"
+#include "Efreet_Mime.h"
 #include "efreet_private.h"
 
 static Eina_List *globs = NULL;     /* contains Efreet_Mime_Glob structs */
@@ -54,7 +53,18 @@ static Eina_Hash *wild = NULL;      /* contains *.ext and mime.types globs*/
 static Eina_Hash *monitors = NULL;  /* contains file monitors */
 static Eina_Hash *mime_icons = NULL; /* contains cache with mime->icons */
 static Eina_Inlist *mime_icons_lru = NULL;
-static unsigned int _init_count = 0;
+static unsigned int _efreet_mime_init_count = 0;
+
+static const char *_mime_inode_symlink = NULL;
+static const char *_mime_inode_fifo = NULL;
+static const char *_mime_inode_chardevice = NULL;
+static const char *_mime_inode_blockdevice = NULL;
+static const char *_mime_inode_socket = NULL;
+static const char *_mime_inode_mountpoint = NULL;
+static const char *_mime_inode_directory = NULL;
+static const char *_mime_application_x_executable = NULL;
+static const char *_mime_application_octet_stream = NULL;
+static const char *_mime_text_plain = NULL;
 
 /**
  * @internal
@@ -139,7 +149,7 @@ struct Efreet_Mime_Icon_Entry_Head
     EINA_INLIST; /* node of mime_icons_lru */
     Eina_Inlist *list;
     const char *mime;
-    time_t timestamp;
+    double timestamp;
 };
 
 typedef struct Efreet_Mime_Icon_Entry Efreet_Mime_Icon_Entry;
@@ -151,6 +161,14 @@ struct Efreet_Mime_Icon_Entry
     unsigned int size;
 };
 
+/* define macros and variable for using the eina logging system  */
+
+#ifdef EFREET_MODULE_LOG_DOM
+#undef EFREET_MODULE_LOG_DOM
+#endif
+#define EFREET_MODULE_LOG_DOM _efreet_mime_log_dom
+static int _efreet_mime_log_dom = -1;
+
 
 static int efreet_mime_glob_remove(const char *glob);
 static void efreet_mime_mime_types_load(const char *file);
@@ -176,7 +194,7 @@ static void efreet_mime_cb_update_file(void *data,
                                         Ecore_File_Event event,
                                         const char *path);
 
-static void efreet_mime_icons_flush(time_t now);
+static void efreet_mime_icons_flush(double now);
 static void efreet_mime_icon_entry_head_free(Efreet_Mime_Icon_Entry_Head *entry);
 static void efreet_mime_icon_entry_add(const char *mime,
                                        const char *icon,
@@ -194,18 +212,26 @@ static void efreet_mime_icons_debug(void);
 EAPI int
 efreet_mime_init(void)
 {
-    _init_count++;
-    if (_init_count > 1)
-        return 1;
+    if (++_efreet_mime_init_count != 1)
+        return _efreet_mime_init_count;
 
     if (!ecore_init())
-        return 0;
+        return --_efreet_mime_init_count;
 
     if (!ecore_file_init())
-        return 0;
+        goto shutdown_ecore;
 
     if (!efreet_init())
-        return 0;
+        goto shutdown_ecore_file;
+
+    _efreet_mime_log_dom = eina_log_domain_register
+      ("efreet_mime", EFREET_DEFAULT_LOG_COLOR);
+
+    if (_efreet_mime_log_dom < 0)
+    {
+        ERROR("Efreet: Could not create a log domain for efreet_mime.");
+        goto shutdown_efreet;
+    }
 
     efreet_mime_endianess = efreet_mime_endian_check();
 
@@ -214,41 +240,60 @@ efreet_mime_init(void)
     efreet_mime_type_cache_clear();
 
     if (!efreet_mime_init_files())
-        return 0;
+        goto unregister_log_domain;
 
-    return 1;
+    return _efreet_mime_init_count;
+
+unregister_log_domain:
+    eina_log_domain_unregister(_efreet_mime_log_dom);
+shutdown_efreet:
+    efreet_shutdown();
+shutdown_ecore_file:
+    ecore_file_shutdown();
+shutdown_ecore:
+    ecore_shutdown();
+
+    return --_efreet_mime_init_count;
 }
 
 /**
  * @return Returns no value
  * @brief Cleans up the efreet mime settings system
  */
-EAPI void
+EAPI int
 efreet_mime_shutdown(void)
 {
-    void *d;
-
-    if (_init_count == 0)
-        return;
-    _init_count--;
-    if (_init_count > 0)
-        return;
+    if (--_efreet_mime_init_count != 0)
+        return _efreet_mime_init_count;
 
     efreet_mime_icons_debug();
 
+    IF_RELEASE(_mime_inode_symlink);
+    IF_RELEASE(_mime_inode_fifo);
+    IF_RELEASE(_mime_inode_chardevice);
+    IF_RELEASE(_mime_inode_blockdevice);
+    IF_RELEASE(_mime_inode_socket);
+    IF_RELEASE(_mime_inode_mountpoint);
+    IF_RELEASE(_mime_inode_directory);
+    IF_RELEASE(_mime_application_x_executable);
+    IF_RELEASE(_mime_application_octet_stream);
+    IF_RELEASE(_mime_text_plain);
+
     IF_FREE_LIST(globs, efreet_mime_glob_free);
     IF_FREE_LIST(magics, efreet_mime_magic_free);
     IF_FREE_HASH(monitors);
     IF_FREE_HASH(wild);
     IF_FREE_HASH(mime_icons);
-
+    eina_log_domain_unregister(_efreet_mime_log_dom);
     efreet_shutdown();
     ecore_file_shutdown();
     ecore_shutdown();
+
+    return _efreet_mime_init_count;
 }
 
 /**
- * @param file: The file to find the mime type
+ * @param file The file to find the mime type
  * @return Returns mime type as a string
  * @brief Retreive the mime type of a file
  */
@@ -257,6 +302,9 @@ efreet_mime_type_get(const char *file)
 {
     const char *type = NULL;
 
+    if (!file)
+      return NULL;
+
     if ((type = efreet_mime_special_check(file)))
         return type;
 
@@ -276,16 +324,16 @@ efreet_mime_type_get(const char *file)
 }
 
 /**
- * @param mime: The name of the mime type
- * @param theme: The name of the theme to search icons in
- * @param size: The wanted size of the icon
+ * @param mime The name of the mime type
+ * @param theme The name of the theme to search icons in
+ * @param size The wanted size of the icon
  * @return Returns mime type icon path as a string
  * @brief Retreive the mime type icon for a file
  */
-EAPI char *
+EAPI const char *
 efreet_mime_type_icon_get(const char *mime, const char *theme, unsigned int size)
 {
-    char *icon = NULL;
+    const char *icon = NULL;
     char *data;
     Eina_List *icons  = NULL;
     const char *env = NULL;
@@ -303,7 +351,7 @@ efreet_mime_type_icon_get(const char *mime, const char *theme, unsigned int size
     {
         eina_stringshare_del(mime);
         eina_stringshare_del(theme);
-        return strdup(cache);
+        return cache;
     }
 
     /* Standard icon name */
@@ -336,6 +384,9 @@ efreet_mime_type_icon_get(const char *mime, const char *theme, unsigned int size
     {
         *ppp = '\0';
 
+        snprintf(buf, sizeof(buf), "%s-x-generic", pp);
+        icons = eina_list_append(icons, strdup(buf));
+
         snprintf(buf, sizeof(buf), "%s-generic", pp);
         icons = eina_list_append(icons, strdup(buf));
 
@@ -358,6 +409,9 @@ efreet_mime_type_icon_get(const char *mime, const char *theme, unsigned int size
     return icon;
 }
 
+/**
+ * @brief Clear mime icons mapping cache
+ */
 EAPI void
 efreet_mime_type_cache_clear(void)
 {
@@ -366,18 +420,24 @@ efreet_mime_type_cache_clear(void)
         eina_hash_free(mime_icons);
         mime_icons_lru = NULL;
     }
-    mime_icons = eina_hash_pointer_new(EINA_FREE_CB(efreet_mime_icon_entry_head_free));
+    mime_icons = eina_hash_stringshared_new(EINA_FREE_CB(efreet_mime_icon_entry_head_free));
 }
 
+/**
+ * @brief Flush mime icons mapping cache
+ *
+ * Flush timeout is defined at compile time by
+ * EFREET_MIME_ICONS_FLUSH_TIMEOUT
+ */
 EAPI void
 efreet_mime_type_cache_flush(void)
 {
-    efreet_mime_icons_flush((time_t)ecore_loop_time_get());
+    efreet_mime_icons_flush(ecore_loop_time_get());
 }
 
 
 /**
- * @param file: The file to check the mime type
+ * @param file The file to check the mime type
  * @return Returns mime type as a string
  * @brief Retreive the mime type of a file using magic
  */
@@ -388,7 +448,7 @@ efreet_mime_magic_type_get(const char *file)
 }
 
 /**
- * @param file: The file to check the mime type
+ * @param file The file to check the mime type
  * @return Returns mime type as a string
  * @brief Retreive the mime type of a file using globs
  */
@@ -402,6 +462,8 @@ efreet_mime_globs_type_get(const char *file)
     char *ext, *mime;
 
     /* Check in the extension hash for the type */
+    if (!file) return NULL;
+
     ext = strchr(file, '.');
     if (ext)
     {
@@ -436,7 +498,7 @@ efreet_mime_globs_type_get(const char *file)
 }
 
 /**
- * @param file: The file to check the mime type
+ * @param file The file to check the mime type
  * @return Returns mime type as a string
  * @brief Retreive the special mime type of a file
  */
@@ -447,7 +509,7 @@ efreet_mime_special_type_get(const char *file)
 }
 
 /**
- * @param file: The file to check the mime type
+ * @param file The file to check the mime type
  * @return Returns mime type as a string
  * @brief Retreive the fallback mime type of a file
  */
@@ -471,7 +533,7 @@ efreet_mime_endian_check(void)
 
 /**
  * @internal
- * @param file: File to monitor
+ * @param file File to monitor
  * @return Returns no value.
  * @brief Creates a new file monitor if we aren't already monitoring the
  * given file
@@ -487,16 +549,16 @@ efreet_mime_monitor_add(const char *file)
         return;
 
     if ((fm = ecore_file_monitor_add(file, efreet_mime_cb_update_file, NULL)))
-      {
-	 eina_hash_del(monitors, file, NULL);
-	 eina_hash_add(monitors, file, fm);
-      }
+    {
+        eina_hash_del(monitors, file, NULL);
+        eina_hash_add(monitors, file, fm);
+    }
 }
 
 /**
  * @internal
- * @param datadirs: List of XDG data dirs
- * @param datahome: Path to XDG data home directory
+ * @param datadirs List of XDG data dirs
+ * @param datahome Path to XDG data home directory
  * @return Returns no value
  * @brief Read all glob files in XDG data/home dirs.
  * Also reads the /etc/mime.types file.
@@ -525,8 +587,8 @@ efreet_mime_load_globs(Eina_List *datadirs, const char *datahome)
     efreet_mime_mime_types_load("/etc/mime.types");
 
     datadir = datahome;
-        snprintf(buf, sizeof(buf), "%s/mime/globs", datadir);
-        efreet_mime_shared_mimeinfo_globs_load(buf);
+    snprintf(buf, sizeof(buf), "%s/mime/globs", datadir);
+    efreet_mime_shared_mimeinfo_globs_load(buf);
 
     EINA_LIST_FOREACH(datadirs, l, datadir)
     {
@@ -537,8 +599,8 @@ efreet_mime_load_globs(Eina_List *datadirs, const char *datahome)
 
 /**
  * @internal
- * @param datadirs: List of XDG data dirs
- * @param datahome: Path to XDG data home directory
+ * @param datadirs List of XDG data dirs
+ * @param datahome Path to XDG data home directory
  * @return Returns no value
  * @brief Read all magic files in XDG data/home dirs.
  */
@@ -556,8 +618,8 @@ efreet_mime_load_magics(Eina_List *datadirs, const char *datahome)
     }
 
     datadir = datahome;
-        snprintf(buf, sizeof(buf), "%s/mime/magic", datadir);
-        efreet_mime_shared_mimeinfo_magic_load(buf);
+    snprintf(buf, sizeof(buf), "%s/mime/magic", datadir);
+    efreet_mime_shared_mimeinfo_magic_load(buf);
 
     EINA_LIST_FOREACH(datadirs, l, datadir)
     {
@@ -568,10 +630,10 @@ efreet_mime_load_magics(Eina_List *datadirs, const char *datahome)
 
 /**
  * @internal
- * @param data: Data pointer passed to monitor_add
- * @param monitor: Ecore_File_Monitor associated with this event
- * @param event: The type of event
- * @param path: Path to the file that was updated
+ * @param data Data pointer passed to monitor_add
+ * @param monitor Ecore_File_Monitor associated with this event
+ * @param event The type of event
+ * @param path Path to the file that was updated
  * @return Returns no value
  * @brief Callback for all file monitors.  Just reloads the appropriate
  * list depending on which file changed.  If it was a magic file
@@ -601,8 +663,8 @@ efreet_mime_cb_update_file(void *data __UNUSED__,
 
 /**
  * @internal
- * @param datadirs: List of XDG data dirs
- * @param datahome: Path to XDG data home directory
+ * @param datadirs List of XDG data dirs
+ * @param datahome Path to XDG data home directory
  * @return Returns 1 on success, 0 on failure
  * @brief Initializes globs, magics, and monitors lists.
  */
@@ -625,12 +687,12 @@ efreet_mime_init_files(void)
      * We watch the directories so we can watch for new files
      */
     datadir = datahome;
-        snprintf(buf, PATH_MAX, "%s/mime", datadir);
-        efreet_mime_monitor_add(buf);
+    snprintf(buf, sizeof(buf), "%s/mime", datadir);
+    efreet_mime_monitor_add(buf);
 
     EINA_LIST_FOREACH(datadirs, l, datadir)
     {
-        snprintf(buf, PATH_MAX, "%s/mime", datadir);
+        snprintf(buf, sizeof(buf), "%s/mime", datadir);
         efreet_mime_monitor_add(buf);
     }
     efreet_mime_monitor_add("/etc/mime.types");
@@ -639,12 +701,23 @@ efreet_mime_init_files(void)
     efreet_mime_load_globs(datadirs, datahome);
     efreet_mime_load_magics(datadirs, datahome);
 
+    _mime_inode_symlink		   = eina_stringshare_add("inode/symlink");
+    _mime_inode_fifo		   = eina_stringshare_add("inode/fifo");
+    _mime_inode_chardevice	   = eina_stringshare_add("inode/chardevice");
+    _mime_inode_blockdevice	   = eina_stringshare_add("inode/blockdevice");
+    _mime_inode_socket		   = eina_stringshare_add("inode/socket");
+    _mime_inode_mountpoint	   = eina_stringshare_add("inode/mountpoint");
+    _mime_inode_directory	   = eina_stringshare_add("inode/directory");
+    _mime_application_x_executable = eina_stringshare_add("application/x-executable");
+    _mime_application_octet_stream = eina_stringshare_add("application/octet-stream");
+    _mime_text_plain               = eina_stringshare_add("text/plain");
+
     return 1;
 }
 
 /**
  * @internal
- * @param file: File to examine
+ * @param file File to examine
  * @return Returns mime type if special file, else NULL
  * @brief Returns a mime type based on the stat of a file.
  * This is used first to catch directories and other special
@@ -671,28 +744,33 @@ efreet_mime_special_check(const char *file)
     struct stat s;
     int path_len = 0;
 
+    /* no link on Windows < Vista */
+#ifdef _WIN32
+    if (!stat(file, &s))
+#else
     if (!lstat(file, &s))
+#endif
     {
         if (S_ISREG(s.st_mode))
             return NULL;
 
 #ifndef _WIN32
         if (S_ISLNK(s.st_mode))
-            return "inode/symlink";
+        return _mime_inode_symlink;
 #endif
 
         if (S_ISFIFO(s.st_mode))
-            return "inode/fifo";
+            return _mime_inode_fifo;
 
         if (S_ISCHR(s.st_mode))
-            return "inode/chardevice";
+            return _mime_inode_chardevice;
 
         if (S_ISBLK(s.st_mode))
-            return "inode/blockdevice";
+            return _mime_inode_blockdevice;
 
 #ifndef _WIN32
         if (S_ISSOCK(s.st_mode))
-            return "inode/socket";
+            return _mime_inode_socket;
 #endif
 
         if (S_ISDIR(s.st_mode))
@@ -712,13 +790,17 @@ efreet_mime_special_check(const char *file)
             /* Truncate to last slash */
             while (parent[--path_len] != '/') parent[path_len] = '\0';
 
+#ifdef _WIN32
+            if (!stat(file, &s2))
+#else
             if (!lstat(parent, &s2))
+#endif
             {
                 if (s.st_dev != s2.st_dev)
-                    return "inode/mount-point";
+                    return _mime_inode_mountpoint;
             }
 
-            return "inode/directory";
+            return _mime_inode_directory;
         }
 
         return NULL;
@@ -729,7 +811,7 @@ efreet_mime_special_check(const char *file)
 
 /**
  * @internal
- * @param file: File to examine
+ * @param file File to examine
  * @return Returns mime type or NULL if the file doesn't exist
  * @brief Returns text/plain if the file appears to contain text and
  * returns application/octet-stream if it appears to be binary.
@@ -741,12 +823,15 @@ efreet_mime_fallback_check(const char *file)
     char buf[32];
     int i;
 
+    if (ecore_file_can_exec(file))
+        return _mime_application_x_executable;
+
     if (!(f = fopen(file, "r"))) return NULL;
 
     i = fread(buf, 1, sizeof(buf), f);
     fclose(f);
 
-    if (i == 0) return "application/octet-stream";
+    if (i == 0) return _mime_application_octet_stream;
 
     /*
      * Check for ASCII control characters in the first 32 bytes.
@@ -759,15 +844,15 @@ efreet_mime_fallback_check(const char *file)
             (buf[i] != '\n') &&     /* Line Feed */
             (buf[i] != '\r') &&     /* Carriage Return */
             (buf[i] != '\t'))       /* Tab */
-            return "application/octet-stream";
+            return _mime_application_octet_stream;
     }
 
-    return "text/plain";
+    return _mime_text_plain;
 }
 
 /**
  * @internal
- * @param glob: Glob to search for
+ * @param glob Glob to search for
  * @return Returns 1 on success, 0 on failure
  * @brief Removes a glob from the list
  */
@@ -777,20 +862,20 @@ efreet_mime_glob_remove(const char *glob)
     Efreet_Mime_Glob *mime = NULL;
 
     if ((mime = eina_list_search_unsorted(globs, EINA_COMPARE_CB(strcmp), glob)))
-        {
+    {
         globs = eina_list_remove(globs, mime);
-            IF_RELEASE(mime->glob);
-            IF_RELEASE(mime->mime);
-            FREE(mime);
-            return 1;
-        }
+        IF_RELEASE(mime->glob);
+        IF_RELEASE(mime->mime);
+        FREE(mime);
+        return 1;
+    }
 
     return 0;
 }
 
 /**
  * @internal
- * @param file: mime.types file to load
+ * @param file mime.types file to load
  * @return Returns no value
  * @brief Loads values from a mime.types style file
  * into the globs list.
@@ -834,7 +919,7 @@ efreet_mime_mime_types_load(const char *file)
             strncpy(ext, pp, (p - pp));
             ext[p - pp] = 0;
 
-	    eina_hash_del(wild, ext, NULL);
+            eina_hash_del(wild, ext, NULL);
             eina_hash_add(wild, ext, (void*)eina_stringshare_add(mimetype));
         }
         while ((*p != '\n') && (*p != 0));
@@ -844,7 +929,7 @@ efreet_mime_mime_types_load(const char *file)
 
 /**
  * @internal
- * @param file: globs file to load
+ * @param file globs file to load
  * @return Returns no value
  * @brief Loads values from a mime.types style file
  * into the globs list.
@@ -894,7 +979,7 @@ efreet_mime_shared_mimeinfo_globs_load(const char *file)
         {
             eina_hash_del(wild, &(ext[2]), NULL);
             eina_hash_add(wild, &(ext[2]),
-			  (void*)eina_stringshare_add(mimetype));
+                            (void*)eina_stringshare_add(mimetype));
         }
         else
         {
@@ -923,7 +1008,7 @@ efreet_mime_shared_mimeinfo_globs_load(const char *file)
 
 /**
  * @internal
- * @param in: Number to count the digits
+ * @param in Number to count the digits
  * @return Returns number of digits
  * @brief Calculates and returns the number of digits
  * in a number.
@@ -941,7 +1026,7 @@ efreet_mime_count_digits(int in)
 
 /**
  * @internal
- * @param file: File to parse
+ * @param file File to parse
  * @return Returns no value
  * @brief Loads a magic file and adds information to magics list
  */
@@ -973,7 +1058,7 @@ efreet_mime_shared_mimeinfo_magic_load(const char *file)
 }
 
 /**
- * @param data: The data from the file
+ * @param data The data from the file
  * @return Returns no value
  * @brief Parses a magic file
  * @note Format:
@@ -1007,7 +1092,7 @@ efreet_mime_shared_mimeinfo_magic_load(const char *file)
  * The indent, range-length, word-size and mask components are optional.
  * If missing, indent defaults to 0, range-length to 1, the word-size to 1,
  * and the mask to all 'one' bits.  In our case, mask is null as it is
- * quicker, uses less memory and will acheive the same exact effect.
+ * quicker, uses less memory and will achieve the same exact effect.
  */
 static void
 efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
@@ -1049,6 +1134,7 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
             ptr = ++val;
 
             while (*ptr != '\n') ptr++;
+            ptr++;
         }
         else
         {
@@ -1070,10 +1156,9 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
                 entry->range_len = 1;
                 entry->mask = NULL;
                 entry->value = NULL;
-                ptr++;
 
                 mime->entries = eina_list_append(mime->entries, entry);
-           }
+            }
 
             switch(*ptr)
             {
@@ -1086,6 +1171,7 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
                 case '=':
                     ptr++;
 
+                    tshort = 0;
                     memcpy(&tshort, ptr, sizeof(short));
                     entry->value_len = ntohs(tshort);
                     ptr += 2;
@@ -1105,11 +1191,10 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
                 case '~':
                     ptr++;
                     entry->word_size = atoi(ptr);
-                    if (((entry->word_size != 0)
-                            && (entry->word_size != 1)
-                            && (entry->word_size != 2)
-                            && (entry->word_size != 4))
-                            || (entry->value_len % entry->word_size))
+                    if ((entry->word_size != 0) && (((entry->word_size != 1)
+                                    && (entry->word_size != 2)
+                                    && (entry->word_size != 4))
+                                || (entry->value_len % entry->word_size)))
                     {
                         /* Invalid, Destroy */
                         FREE(entry->value);
@@ -1183,9 +1268,9 @@ efreet_mime_shared_mimeinfo_magic_parse(char *data, int size)
 
 /**
  * @internal
- * @param file: File to check
- * @param start: Start priority, if 0 start at beginning
- * @param end: End priority, should be less then start
+ * @param file File to check
+ * @param start Start priority, if 0 start at beginning
+ * @param end End priority, should be less then start
  * unless start
  * @return Returns mime type for file if found, NULL if not
  * @brief Applies magic rules to a file given a start and end priority
@@ -1237,7 +1322,7 @@ efreet_mime_magic_check_priority(const char *file,
             else if ((level > e->indent) && match)
             {
                 fclose(f);
-                if (last_mime) return last_mime;
+                return last_mime;
             }
 
             for (offset = e->offset; offset < e->offset + e->range_len; offset++)
@@ -1272,6 +1357,12 @@ efreet_mime_magic_check_priority(const char *file,
                 }
             }
         }
+
+        if (match)
+        {
+            fclose(f);
+            return last_mime;
+        }
     }
     fclose(f);
 
@@ -1280,7 +1371,7 @@ efreet_mime_magic_check_priority(const char *file,
 
 /**
  * @internal
- * @param data: Data pointer that is being destroyed
+ * @param data Data pointer that is being destroyed
  * @return Returns no value
  * @brief Callback for globs destroy
  */
@@ -1296,7 +1387,7 @@ efreet_mime_glob_free(void *data)
 
 /**
  * @internal
- * @param data: Data pointer that is being destroyed
+ * @param data Data pointer that is being destroyed
  * @return Returns no value
  * @brief Callback for magics destroy
  */
@@ -1304,21 +1395,15 @@ static void
 efreet_mime_magic_free(void *data)
 {
     Efreet_Mime_Magic *m = data;
-    Efreet_Mime_Magic_Entry *entry = NULL;
 
     IF_RELEASE(m->mime);
-    while (m->entries)
-    {
-        entry = eina_list_data_get(m->entries);
-        efreet_mime_magic_entry_free(entry);
-        m->entries = eina_list_remove_list(m->entries, m->entries);
-    }
+    IF_FREE_LIST(m->entries, efreet_mime_magic_entry_free);
     IF_FREE(m);
 }
 
 /**
  * @internal
- * @param data: Data pointer that is being destroyed
+ * @param data Data pointer that is being destroyed
  * @return Returns no value
  * @brief Callback for magic entry destroy
  */
@@ -1335,8 +1420,8 @@ efreet_mime_magic_entry_free(void *data)
 
 /**
  * @internal
- * @param str: String (filename) to match
- * @param glob: Glob to match str to
+ * @param str String (filename) to match
+ * @param glob Glob to match str to
  * @return Returns 1 on success, 0 on failure
  * @brief Compares str to glob, case sensitive
  */
@@ -1355,8 +1440,8 @@ efreet_mime_glob_match(const char *str, const char *glob)
 
 /**
  * @internal
- * @param str: String (filename) to match
- * @param glob: Glob to match str to
+ * @param str String (filename) to match
+ * @param glob Glob to match str to
  * @return Returns 1 on success, 0 on failure
  * @brief Compares str to glob, case insensitive (expects str already in lower case)
  */
@@ -1380,10 +1465,10 @@ efreet_mime_glob_case_match(char *str, const char *glob)
 }
 
 static void
-efreet_mime_icons_flush(time_t now)
+efreet_mime_icons_flush(double now)
 {
     Eina_Inlist *l;
-    static time_t old = 0;
+    static double old = 0;
     int todo;
 
     if (now - old < EFREET_MIME_ICONS_FLUSH_TIMEOUT)
@@ -1492,7 +1577,7 @@ efreet_mime_icon_entry_add(const char *mime,
         mime_icons_lru = eina_inlist_prepend(mime_icons_lru, l);
     }
 
-    entry->timestamp = (time_t)ecore_loop_time_get();
+    entry->timestamp = ecore_loop_time_get();
     efreet_mime_icons_flush(entry->timestamp);
 }
 
@@ -1522,7 +1607,7 @@ efreet_mime_icon_entry_find(const char *mime,
             if (mime_icons_lru != l)
                 mime_icons_lru = eina_inlist_promote(mime_icons_lru, l);
 
-            entry->timestamp = (time_t)ecore_loop_time_get();
+            entry->timestamp = ecore_loop_time_get();
             return n->icon;
         }
     }
@@ -1534,7 +1619,7 @@ efreet_mime_icon_entry_find(const char *mime,
 static void
 efreet_mime_icons_debug(void)
 {
-    time_t now = (time_t)ecore_loop_time_get();
+    double now = ecore_loop_time_get();
     Efreet_Mime_Icon_Entry_Head *entry;
     EINA_INLIST_FOREACH(mime_icons_lru, entry)
     {
@@ -1547,11 +1632,11 @@ efreet_mime_icons_debug(void)
             now = 0;
         }
 
-        printf("mime-icon entry: '%s' last used: %s",
-               entry->mime, ctime(&entry->timestamp));
+        DBG("mime-icon entry: '%s' last used: %s",
+            entry->mime, ctime(&entry->timestamp));
 
         EINA_INLIST_FOREACH(entry->list, n)
-            printf("\tsize: %3u theme: '%s' icon: '%s'\n",
+            DBG("\tsize: %3u theme: '%s' icon: '%s'",
                    n->theme, n->size, n->icon);
     }
 }
